@@ -4,7 +4,6 @@ import dataclasses
 import logging
 import shlex
 import sys
-import uuid
 import webbrowser
 from datetime import datetime, timedelta, timezone
 from typing import AsyncIterator, List, Optional, Sequence, Set, Tuple
@@ -26,7 +25,6 @@ from neuro_sdk import (
     JobStatus,
     Permission,
     RemoteImage,
-    Volume,
 )
 
 from neuro_cli.formatters.images import DockerImageProgress
@@ -41,6 +39,7 @@ from neuro_cli.utils import resolve_disk
 
 from .ael import process_attach, process_exec, process_logs
 from .click_types import (
+    CLUSTER,
     JOB,
     JOB_COLUMNS,
     JOB_NAME,
@@ -292,6 +291,7 @@ async def attach(root: Root, job: str, port_forward: List[Tuple[int, int]]) -> N
 )
 @option(
     "--cluster",
+    type=CLUSTER,
     help="Show jobs on a specified cluster (the current cluster by default).",
 )
 @option(
@@ -406,35 +406,6 @@ async def ls(
     tags = set(tag)
     if not cluster:
         cluster = root.client.config.cluster_name
-    jobs = root.client.jobs.list(
-        statuses=statuses,
-        name=name,
-        owners=owners,
-        tags=tags,
-        since=_parse_date(since),
-        until=_parse_date(until),
-        reverse=recent_first,
-        cluster_name=cluster,
-    )
-
-    # client-side filtering
-    if description:
-        jobs = (job async for job in jobs if job.description == description)
-
-    if distinct:
-
-        async def _filter_distinct(
-            jobs_iter: AsyncIterator[JobDescription],
-        ) -> AsyncIterator[JobDescription]:
-            names: Set[str] = set()
-            async for job in jobs_iter:
-                if job.name in names:
-                    continue
-                if job.name is not None:
-                    names.add(job.name)
-                yield job
-
-        jobs = _filter_distinct(jobs)
 
     uri_fmtr: URIFormatter
     if full_uri:
@@ -454,11 +425,40 @@ async def ls(
             datetime_formatter=get_datetime_formatter(root.iso_datetime_format),
         )
 
-    with root.status("Fetching jobs") as rich_status:
-        jobs_list = []
-        async for job in jobs:
-            jobs_list.append(job)
-            rich_status.update(f"Fetching jobs ({len(jobs_list)} loaded)")
+    async with root.client.jobs.list(
+        statuses=statuses,
+        name=name,
+        owners=owners,
+        tags=tags,
+        since=_parse_date(since),
+        until=_parse_date(until),
+        reverse=recent_first,
+        cluster_name=cluster,
+    ) as jobs:
+        # client-side filtering
+        if description:
+            jobs = (job async for job in jobs if job.description == description)
+
+        if distinct:
+
+            async def _filter_distinct(
+                jobs_iter: AsyncIterator[JobDescription],
+            ) -> AsyncIterator[JobDescription]:
+                names: Set[str] = set()
+                async for job in jobs_iter:
+                    if job.name in names:
+                        continue
+                    if job.name is not None:
+                        names.add(job.name)
+                    yield job
+
+            jobs = _filter_distinct(jobs)
+
+        with root.status("Fetching jobs") as rich_status:
+            jobs_list = []
+            async for job in jobs:
+                jobs_list.append(job)
+                rich_status.update(f"Fetching jobs ({len(jobs_list)} loaded)")
 
     with root.pager():
         root.print(formatter(jobs_list))
@@ -550,6 +550,7 @@ async def browse(root: Root, job: str) -> None:
 @argument("jobs", nargs=-1, required=False, type=JOB)
 @option(
     "--cluster",
+    type=CLUSTER,
     help="Show jobs on a specified cluster (the current cluster by default).",
 )
 @option(
@@ -699,7 +700,7 @@ async def top(
         async def create_pollers() -> None:
             nonlocal since_dt
             while True:
-                jobs = root.client.jobs.list(
+                async with root.client.jobs.list(
                     statuses=JobStatus.active_items(),
                     name=name,
                     owners=owners,
@@ -707,34 +708,36 @@ async def top(
                     since=since_dt,
                     until=until_dt,
                     cluster_name=cluster,
-                )
+                ) as jobs:
+                    # client-side filtering
+                    if description:
+                        jobs = (
+                            job async for job in jobs if job.description == description
+                        )
 
-                # client-side filtering
-                if description:
-                    jobs = (job async for job in jobs if job.description == description)
-
-                dt: Optional[datetime]
-                dt = datetime.now(timezone.utc) - timedelta(minutes=1)
-                if since_dt is None or since_dt < dt:
-                    since_dt = dt
-                async for job in jobs:
-                    job_id = job.id
-                    if job_id in observed:
-                        continue
-                    observed.add(job_id)
-                    dt = job.history.created_at
-                    if dt is not None and since_dt < dt:
+                    dt: Optional[datetime]
+                    dt = datetime.now(timezone.utc) - timedelta(minutes=1)
+                    if since_dt is None or since_dt < dt:
                         since_dt = dt
-                    loop.create_task(poller(job))
+                    async for job in jobs:
+                        job_id = job.id
+                        if job_id in observed:
+                            continue
+                        observed.add(job_id)
+                        dt = job.history.created_at
+                        if dt is not None and since_dt < dt:
+                            since_dt = dt
+                        loop.create_task(poller(job))
                 await asyncio.sleep(TOP_NEW_JOBS_DELAY)
 
     async def poller(job: JobDescription) -> None:
         try:
-            async for info in root.client.jobs.top(
+            async with root.client.jobs.top(
                 job.id, cluster_name=job.cluster_name
-            ):
-                formatter.update(job, info)
-                await asyncio.sleep(0)
+            ) as it:
+                async for info in it:
+                    formatter.update(job, info)
+                    await asyncio.sleep(0)
         except ValueError:
             pass  # Job is finished.
         finally:
@@ -823,6 +826,12 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
 @command(context_settings=dict(allow_interspersed_args=False))
 @argument("image", type=RemoteImageType())
 @argument("cmd", nargs=-1, type=click.UNPROCESSED)
+@option(
+    "--cluster",
+    type=CLUSTER,
+    help="Run job in a specified cluster",
+    default=None,
+)
 @option(
     "-s",
     "--preset",
@@ -1014,6 +1023,7 @@ async def run(
     root: Root,
     image: RemoteImage,
     preset: str,
+    cluster: Optional[str],
     extshm: bool,
     http: int,
     http_auth: Optional[bool],
@@ -1060,9 +1070,11 @@ async def run(
     # registry, run /script.sh and pass arg1 and arg2 as its arguments:
     neuro run -s cpu-small image:my-ubuntu:latest --entrypoint=/script.sh arg1 arg2
     """
+    cluster_name = cluster or root.client.cluster_name
+    cluster_config = root.client.config.clusters[cluster_name]
     if not preset:
-        preset = next(iter(root.client.config.presets.keys()))
-    job_preset = root.client.config.presets[preset]
+        preset = next(iter(cluster_config.presets.keys()))
+    job_preset = cluster_config.presets[preset]
     if preemptible is not None:
         root.print(
             "-p/-P option is deprecated and ignored. Use corresponding presets instead."
@@ -1098,7 +1110,7 @@ async def run(
         schedule_timeout=schedule_timeout,
         privileged=privileged,
         share=share,
-        cluster_name=root.client.cluster_name,
+        cluster_name=cluster or root.client.cluster_name,
     )
 
 
@@ -1204,7 +1216,7 @@ async def run_job(
         job_schedule_timeout = parse_timedelta(schedule_timeout).total_seconds()
     log.debug(f"Job schedule timeout: {job_schedule_timeout}")
 
-    env_parse_result = root.client.parse.envs(env, env_file)
+    env_parse_result = root.client.parse.envs(env, env_file, cluster_name=cluster_name)
     env_dict, secret_env_dict = env_parse_result.env, env_parse_result.secret_env
     real_cmd = _parse_cmd(cmd)
 
@@ -1213,13 +1225,15 @@ async def run_job(
 
     log.info(f"Using image '{image}'")
 
-    volume_parse_result = root.client.parse.volumes(volume)
+    volume_parse_result = root.client.parse.volumes(volume, cluster_name=cluster_name)
     volumes = list(volume_parse_result.volumes)
     secret_files = volume_parse_result.secret_files
 
     # Replace disk names with disk ids
     async def _force_disk_id(disk_uri: URL) -> URL:
-        disk_id = await resolve_disk(disk_uri.parts[-1], client=root.client)
+        disk_id = await resolve_disk(
+            disk_uri.parts[-1], client=root.client, cluster_name=cluster_name
+        )
         return disk_uri / f"../{disk_id}"
 
     disk_volumes = [
@@ -1234,19 +1248,6 @@ async def run_job(
         if env_name in env_dict:
             raise ValueError(f"{env_name} is already set to {env_dict[env_name]}")
 
-        # The following code is compatibility layer with old images
-        # TODO: remove this and upload_and_map_config function
-        old_env_name = "NEURO_STEAL_CONFIG"
-        if old_env_name in env_dict:
-            raise ValueError(
-                f"{old_env_name} is already set to {env_dict[old_env_name]}"
-            )
-
-        env_var, secret_volume = await upload_and_map_config(root, cluster_name)
-        env_dict[old_env_name] = env_var
-        volumes.append(secret_volume)
-        # End of compatibility layer
-
     if volumes:
         log.info(
             "Using volumes: \n"
@@ -1256,6 +1257,7 @@ async def run_job(
     job = await root.client.jobs.start(
         image=image,
         preset_name=preset,
+        cluster_name=cluster_name,
         entrypoint=entrypoint,
         command=real_cmd,
         working_dir=working_dir,
@@ -1316,38 +1318,6 @@ def _parse_cmd(cmd: Sequence[str]) -> str:
     return real_cmd
 
 
-async def upload_and_map_config(root: Root, cluster_name: str) -> Tuple[str, Volume]:
-
-    # store the Neuro CLI config on the storage under some random path
-    nmrc_path = URL(root.config_path.expanduser().resolve().as_uri())
-    random_nmrc_filename = f"{uuid.uuid4()}-cfg"
-    storage_nmrc_folder = URL(
-        f"storage://{cluster_name}/{root.client.username}/.neuro/"
-    )
-    storage_nmrc_path = storage_nmrc_folder / random_nmrc_filename
-    local_nmrc_folder = f"{STORAGE_MOUNTPOINT}/.neuro/"
-    local_nmrc_path = f"{local_nmrc_folder}{random_nmrc_filename}"
-    if not root.quiet:
-        root.print(f"Temporary config file created on storage: {storage_nmrc_path}.")
-        root.print(f"Inside container it will be available at: {local_nmrc_path}.")
-    await root.client.storage.mkdir(storage_nmrc_folder, parents=True, exist_ok=True)
-
-    async def skip_tmp(fname: str) -> bool:
-        return not fname.endswith(("-shm", "-wal", "-journal"))
-
-    await root.client.storage.upload_dir(nmrc_path, storage_nmrc_path, filter=skip_tmp)
-    # specify a container volume and mount the storage path
-    # into specific container path
-    return (
-        local_nmrc_path,
-        Volume(
-            storage_uri=storage_nmrc_folder,
-            container_path=local_nmrc_folder,
-            read_only=False,
-        ),
-    )
-
-
 async def browse_job(root: Root, job: JobDescription) -> None:
     url = job.http_url
     if url.scheme not in ("http", "https"):
@@ -1443,8 +1413,6 @@ def _job_to_cli_args(job: JobDescription) -> List[str]:
     if job.description:
         res += ["--description", shlex.quote(job.description)]
     for volume in job.container.volumes:
-        if volume.container_path == "/var/storage/.neuro" and job.pass_config:
-            continue
         res += [
             "--volume",
             (
@@ -1464,7 +1432,7 @@ def _job_to_cli_args(job: JobDescription) -> List[str]:
     if job.container.working_dir:
         res += ["--workdir", job.container.working_dir]
     for env_name, env_value in job.container.env.items():
-        if env_name in (PASS_CONFIG_ENV_NAME, "NEURO_STEAL_CONFIG") and job.pass_config:
+        if env_name == PASS_CONFIG_ENV_NAME and job.pass_config:
             continue  # Do not specify value for pass config env variable
         res += ["--env", f"{env_name}={env_value}"]
     for env_name, secret_uri in job.container.secret_env.items():
