@@ -460,15 +460,16 @@ async def resolve_job_ex(
         return id_or_name, cluster_name
 
     try:
-        async for job in client.jobs.list(
+        async with client.jobs.list(
             name=id_or_name,
             owners={owner},
             reverse=True,
             limit=1,
             cluster_name=cluster_name,
-        ):
-            log.debug(f"Job name '{id_or_name}' resolved to job ID '{job.id}'")
-            return job.id, cluster_name
+        ) as it:
+            async for job in it:
+                log.debug(f"Job name '{id_or_name}' resolved to job ID '{job.id}'")
+                return job.id, cluster_name
     except asyncio.CancelledError:
         raise
     except ClientResponseError as e:
@@ -485,12 +486,14 @@ async def resolve_job_ex(
 DISK_ID_PATTERN = r"disk-[0-9a-z]{8}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{12}"
 
 
-async def resolve_disk(id_or_name: str, *, client: Client) -> str:
+async def resolve_disk(
+    id_or_name: str, *, client: Client, cluster_name: Optional[str] = None
+) -> str:
     # Temporary fast path.
     if re.fullmatch(DISK_ID_PATTERN, id_or_name):
         return id_or_name
 
-    disk = await client.disks.get(id_or_name)
+    disk = await client.disks.get(id_or_name, cluster_name)
     return disk.id
 
 
