@@ -449,7 +449,7 @@ def test_e2e_ssh_exec_false(helper: Helper) -> None:
                 "--timeout",
                 str(EXEC_TIMEOUT),
                 job_id,
-                "false",
+                "bash -c 'echo test_e2e_ssh_exec_false; false'",
             ]
         )
     assert cm.value.returncode == 1
@@ -482,22 +482,30 @@ def test_e2e_ssh_exec_echo(helper: Helper) -> None:
     command = 'bash -c "sleep 15m; false"'
     job_id = helper.run_job_and_wait_state(UBUNTU_IMAGE_NAME, command)
 
-    captured = helper.run_cli(
-        [
-            "--quiet",
-            "job",
-            "exec",
-            "--no-tty",
-            "--no-key-check",
-            "--timeout",
-            str(EXEC_TIMEOUT),
-            job_id,
-            'bash -c "sleep 30; echo ok"',
-        ]
-    )
-    assert captured.err == ""
-    assert captured.out == "ok"
-    helper.kill_job(job_id, wait=False)
+    for i in range(10):
+        captured = helper.run_cli(
+            [
+                "--quiet",
+                "job",
+                "exec",
+                "--no-tty",
+                "--no-key-check",
+                "--timeout",
+                str(EXEC_TIMEOUT),
+                job_id,
+                'bash -c "sleep 30; echo ok"',
+            ]
+        )
+        if captured.out == "":
+            continue
+        assert captured.err == ""
+        assert captured.out == "ok"
+        helper.kill_job(job_id, wait=False)
+        break
+    else:
+        raise AssertionError(
+            f"Cannot exec echo after 10 tries, the last output is {captured}"
+        )
 
 
 @pytest.mark.e2e
@@ -930,7 +938,7 @@ def try_job_top(helper: Helper, *args: str) -> Iterator[str]:
         assert excinfo.value.returncode == 124
         i = stdout.rfind(" ID/NAME ")
         yield (stdout if i < 0 else stdout[i:])
-        if time() - t0 > 10 * 60:
+        if time() - t0 > 20 * 60:
             # timeout is reached without info from server
             raise AssertionError(
                 f"Cannot get response from server "
@@ -943,7 +951,7 @@ def try_job_top(helper: Helper, *args: str) -> Iterator[str]:
 
 @pytest.mark.e2e
 def test_e2e_job_top(helper: Helper) -> None:
-    command = "sleep 400"
+    command = "bash -c 'sleep 1000; echo test_e2e_job_top'"
 
     print("Run job... ")
     job_id = helper.run_job_and_wait_state(image=UBUNTU_IMAGE_NAME, command=command)
@@ -968,7 +976,7 @@ def test_e2e_job_top(helper: Helper) -> None:
 def test_e2e_job_top_filtering(helper: Helper, loop: AbstractEventLoop) -> None:
     job_name = f"test-job-{str(uuid4())[:8]}"
     description = str(uuid4())
-    command = "sleep 1000"
+    command = "bash -c 'sleep 1000; echo test_e2e_job_top_filtering'"
 
     print("Run jobs... ")
     job1_id = helper.run_job_and_wait_state(
@@ -1067,7 +1075,7 @@ def test_e2e_restart_failing(request: Any, helper: Helper) -> None:
             "on-failure",
             "--detach",
             UBUNTU_IMAGE_NAME,
-            "false",
+            "bash -c 'echo test_e2e_restart_failing; false'",
         ]
     )
     job_id = captured.out
@@ -1144,7 +1152,7 @@ def test_job_run_stdout(helper: Helper) -> None:
 def test_job_attach_tty(helper: Helper) -> None:
     job_id = helper.run_job_and_wait_state(
         UBUNTU_IMAGE_NAME,
-        "timeout 300 bash --norc",
+        "timeout 1000 bash --norc",
         tty=True,
         env={"PS1": "# "},
     )
