@@ -7,7 +7,6 @@ from pathlib import Path, PurePath
 from typing import Tuple
 
 import pytest
-from yarl import URL
 
 from neuro_cli.const import EX_OSFILE
 from neuro_cli.formatters.storage import TreeFormatter
@@ -69,26 +68,26 @@ def test_ls_directory_itself(helper: Helper) -> None:
 
 @pytest.mark.e2e
 def test_e2e_mkdir(helper: Helper) -> None:
-    helper.run_cli(["storage", "mkdir", "--parents", helper.tmpstorage + "folder"])
+    helper.run_cli(["storage", "mkdir", "--parents", helper.tmpstorage / "folder"])
     helper.check_dir_exists_on_storage("folder", "")
 
     # Create existing directory
     with pytest.raises(subprocess.CalledProcessError) as cm:
-        helper.run_cli(["storage", "mkdir", helper.tmpstorage + "folder"])
+        helper.run_cli(["storage", "mkdir", helper.tmpstorage / "folder"])
     assert cm.value.returncode == EX_OSFILE
     helper.mkdir("folder", exist_ok=True)
 
     # Create a subdirectory in existing directory
-    helper.run_cli(["storage", "mkdir", helper.tmpstorage + "folder/subfolder"])
+    helper.run_cli(["storage", "mkdir", helper.tmpstorage / "folder/subfolder"])
     helper.check_dir_exists_on_storage("subfolder", "folder")
 
     # Create a subdirectory in non-existing directory
     with pytest.raises(subprocess.CalledProcessError) as cm:
-        helper.run_cli(["storage", "mkdir", helper.tmpstorage + "parent/child"])
+        helper.run_cli(["storage", "mkdir", helper.tmpstorage / "parent/child"])
     assert cm.value.returncode == EX_OSFILE
     helper.check_dir_absent_on_storage("parent", "")
     helper.run_cli(
-        ["storage", "mkdir", "--parents", helper.tmpstorage + "parent/child"]
+        ["storage", "mkdir", "--parents", helper.tmpstorage / "parent/child"]
     )
     helper.check_dir_exists_on_storage("parent", "")
     helper.check_dir_exists_on_storage("child", "parent")
@@ -101,7 +100,7 @@ def test_copy_local_file_to_platform_directory(helper: Helper, data2: _Data) ->
 
     helper.mkdir("folder", parents=True)
     # Upload local file to existing directory
-    helper.run_cli(["storage", "cp", srcfile, helper.tmpstorage + "/folder"])
+    helper.run_cli(["storage", "cp", srcfile, helper.tmpstorage / "folder"])
 
     # Ensure file is there
     helper.check_file_exists_on_storage(file_name, "folder", FILE_SIZE_B // 3)
@@ -116,7 +115,7 @@ def test_copy_local_file_to_platform_directory_explicit(
 
     helper.mkdir("folder", parents=True)
     # Upload local file to existing directory
-    helper.run_cli(["storage", "cp", "-t", helper.tmpstorage + "/folder", srcfile])
+    helper.run_cli(["storage", "cp", "-t", helper.tmpstorage / "folder", srcfile])
 
     # Ensure file is there
     helper.check_file_exists_on_storage(file_name, "folder", FILE_SIZE_B // 3)
@@ -131,7 +130,7 @@ def test_copy_local_single_file_to_platform_file(helper: Helper, data: _Data) ->
     helper.mkdir("folder", parents=True)
     # Upload local file to platform
     helper.run_cli(
-        ["storage", "cp", srcfile, helper.tmpstorage + "/folder/different_name.txt"]
+        ["storage", "cp", srcfile, helper.tmpstorage / "folder/different_name.txt"]
     )
 
     # Ensure file is there
@@ -155,7 +154,7 @@ def test_copy_local_single_file_to_platform_file_explicit(
             "cp",
             "-T",
             srcfile,
-            helper.tmpstorage + "/folder/different_name.txt",
+            helper.tmpstorage / "folder/different_name.txt",
         ]
     )
 
@@ -176,7 +175,7 @@ def test_copy_local_to_platform_single_file_3(helper: Helper, data: _Data) -> No
     # Upload local file to non existing directory
     with pytest.raises(subprocess.CalledProcessError, match=str(EX_OSFILE)):
         captured = helper.run_cli(
-            ["storage", "cp", srcfile, helper.tmpstorage + "/non_existing_dir/"]
+            ["storage", "cp", srcfile, helper.tmpstorage / "non_existing_dir/"]
         )
         assert not captured.err
         assert captured.out == ""
@@ -195,7 +194,7 @@ def test_e2e_copy_non_existing_platform_to_non_existing_local(
             [
                 "storage",
                 "cp",
-                helper.tmpstorage + "/not-exist-foo",
+                helper.tmpstorage / "not-exist-foo",
                 str(tmp_path / "not-exist-bar"),
             ]
         )
@@ -207,7 +206,7 @@ def test_e2e_copy_non_existing_platform_to_____existing_local(
 ) -> None:
     # Try downloading non existing file
     with pytest.raises(subprocess.CalledProcessError, match=str(EX_OSFILE)):
-        helper.run_cli(["storage", "cp", helper.tmpstorage + "/foo", str(tmp_path)])
+        helper.run_cli(["storage", "cp", helper.tmpstorage / "foo", str(tmp_path)])
 
 
 @pytest.mark.e2e
@@ -277,8 +276,8 @@ def test_copy_and_remove_multiple_files(
         [
             "storage",
             "cp",
-            f"{helper.tmpstorage}/{srcname}",
-            f"{helper.tmpstorage}/{srcname2}",
+            helper.tmpstorage / srcname,
+            helper.tmpstorage / srcname2,
             str(targetdir),
         ]
     )
@@ -290,8 +289,8 @@ def test_copy_and_remove_multiple_files(
         [
             "storage",
             "rm",
-            f"{helper.tmpstorage}/{srcname}",
-            f"{helper.tmpstorage}/{srcname2}",
+            helper.tmpstorage / srcname,
+            helper.tmpstorage / srcname2,
         ]
     )
     assert captured.out == ""
@@ -322,7 +321,7 @@ def test_e2e_copy_recursive_to_platform(
     # Download into local directory and confirm checksum
     targetdir = tmp_path / "bar"
     targetdir.mkdir()
-    helper.run_cli(["storage", "cp", "-r", f"{helper.tmpstorage}", str(targetdir)])
+    helper.run_cli(["storage", "cp", "-r", "-T", helper.tmpstorage, str(targetdir)])
     targetfile = targetdir / "nested" / "directory" / "for" / "test" / target_file_name
     print("source file", srcfile)
     print("target file", targetfile)
@@ -340,7 +339,7 @@ def test_e2e_copy_recursive_file(helper: Helper, tmp_path: Path) -> None:
     assert not captured.out
 
     captured = helper.run_cli(
-        ["storage", "cp", "-r", helper.tmpstorage + "testfile", str(dstfile)]
+        ["storage", "cp", "-r", helper.tmpstorage / "testfile", str(dstfile)]
     )
     assert not captured.out
 
@@ -354,8 +353,8 @@ def test_e2e_rename(helper: Helper) -> None:
         [
             "storage",
             "mv",
-            helper.tmpstorage + "folder",
-            helper.tmpstorage + "otherfolder",
+            helper.tmpstorage / "folder",
+            helper.tmpstorage / "otherfolder",
         ]
     )
     helper.check_dir_absent_on_storage("folder", "")
@@ -370,8 +369,8 @@ def test_e2e_move_to_directory(helper: Helper) -> None:
         [
             "storage",
             "mv",
-            helper.tmpstorage + "folder",
-            helper.tmpstorage + "otherfolder",
+            helper.tmpstorage / "folder",
+            helper.tmpstorage / "otherfolder",
         ]
     )
     helper.check_dir_absent_on_storage("folder", "")
@@ -388,8 +387,8 @@ def test_e2e_move_to_directory_explicitly(helper: Helper) -> None:
             "storage",
             "mv",
             "-t",
-            helper.tmpstorage + "otherfolder",
-            helper.tmpstorage + "folder",
+            helper.tmpstorage / "otherfolder",
+            helper.tmpstorage / "folder",
         ]
     )
     helper.check_dir_absent_on_storage("folder", "")
@@ -406,8 +405,8 @@ def test_e2e_move_content_to_directory(helper: Helper) -> None:
             "storage",
             "mv",
             "-T",
-            helper.tmpstorage + "folder",
-            helper.tmpstorage + "otherfolder",
+            helper.tmpstorage / "folder",
+            helper.tmpstorage / "otherfolder",
         ]
     )
     helper.check_dir_absent_on_storage("folder", "")
@@ -443,9 +442,9 @@ def test_e2e_move_target_directory_no_target_directory(helper: Helper) -> None:
                 "storage",
                 "mv",
                 "-t",
-                helper.tmpstorage + "/foo",
+                helper.tmpstorage / "foo",
                 "-T",
-                helper.tmpstorage + "/bar",
+                helper.tmpstorage / "bar",
             ]
         )
     assert "Cannot combine" in cm.value.stderr
@@ -459,9 +458,9 @@ def test_e2e_move_no_target_directory_extra_operand(helper: Helper) -> None:
                 "storage",
                 "mv",
                 "-T",
-                helper.tmpstorage + "/foo",
-                helper.tmpstorage + "/bar",
-                helper.tmpstorage + "/baz",
+                helper.tmpstorage / "foo",
+                helper.tmpstorage / "bar",
+                helper.tmpstorage / "baz",
             ]
         )
     assert "Extra operand after " in cm.value.stderr
@@ -483,10 +482,10 @@ def test_e2e_glob(tmp_path: Path, helper: Helper) -> None:
             "cp",
             "-r",
             tmp_path.as_uri() + "/f*",
-            helper.tmpstorage + "/folder",
+            helper.tmpstorage / "folder",
         ]
     )
-    captured = helper.run_cli(["storage", "ls", helper.tmpstorage + "/folder"])
+    captured = helper.run_cli(["storage", "ls", helper.tmpstorage / "folder"])
     assert sorted(captured.out.splitlines()) == ["bar", "baz", "foo", "subfolder"]
 
     # Move files with pattern
@@ -494,39 +493,35 @@ def test_e2e_glob(tmp_path: Path, helper: Helper) -> None:
         [
             "storage",
             "mv",
-            helper.tmpstorage + "/folder/[bf]*",
-            helper.tmpstorage + "/folder/subfolder",
+            helper.tmpstorage / "folder/[bf]*",
+            helper.tmpstorage / "folder/subfolder",
         ]
     )
-    captured = helper.run_cli(
-        ["storage", "ls", helper.tmpstorage + "/folder/subfolder"]
-    )
+    captured = helper.run_cli(["storage", "ls", helper.tmpstorage / "folder/subfolder"])
     assert sorted(captured.out.splitlines()) == ["bar", "baz", "foo"]
 
     # Download files with pattern
     download = tmp_path / "download"
     download.mkdir()
-    helper.run_cli(["storage", "cp", helper.tmpstorage + "/**/b*", str(download)])
+    helper.run_cli(["storage", "cp", helper.tmpstorage / "**/b*", str(download)])
     assert sorted(download.iterdir()) == [download / "bar", download / "baz"]
 
     # Remove files with pattern
-    helper.run_cli(["storage", "rm", helper.tmpstorage + "/**/b*"])
-    captured = helper.run_cli(
-        ["storage", "ls", helper.tmpstorage + "/folder/subfolder"]
-    )
+    helper.run_cli(["storage", "rm", helper.tmpstorage / "**/b*"])
+    captured = helper.run_cli(["storage", "ls", helper.tmpstorage / "folder/subfolder"])
     assert sorted(captured.out.splitlines()) == ["foo"]
 
     # Test subcommand "glob"
-    captured = helper.run_cli(["storage", "glob", helper.tmpstorage + "/**"])
+    captured = helper.run_cli(["storage", "glob", helper.tmpstorage / "**"])
     prefix = (
         f"storage://{helper.cluster_name}/{helper.username}/"
-        f"{URL(helper.tmpstorage).path}"
+        f"{helper.tmpstorage.path}"
     )
     assert sorted(captured.out.splitlines()) == [
-        prefix,
-        prefix + "folder",
-        prefix + "folder/subfolder",
-        prefix + "folder/subfolder/foo",
+        prefix.rstrip("/"),
+        prefix + "/folder",
+        prefix + "/folder/subfolder",
+        prefix + "/folder/subfolder/foo",
     ]
 
 
@@ -545,13 +540,13 @@ def test_e2e_no_glob(tmp_path: Path, helper: Helper) -> None:
             "-r",
             "--no-glob",
             tmp_path.as_uri() + "/[d]",
-            helper.tmpstorage + "/d",
+            helper.tmpstorage / "d",
         ]
     )
 
     # Move files with literal path
     helper.run_cli(
-        ["storage", "mv", "--no-glob", helper.tmpstorage + "/d/[df]", helper.tmpstorage]
+        ["storage", "mv", "--no-glob", helper.tmpstorage / "d/[df]", helper.tmpstorage]
     )
     captured = helper.run_cli(["storage", "ls", helper.tmpstorage])
     assert sorted(captured.out.splitlines()) == ["[df]", "d"]
@@ -560,12 +555,12 @@ def test_e2e_no_glob(tmp_path: Path, helper: Helper) -> None:
     download = tmp_path / "download"
     download.mkdir()
     helper.run_cli(
-        ["storage", "cp", "--no-glob", helper.tmpstorage + "/[df]", str(download)]
+        ["storage", "cp", "--no-glob", helper.tmpstorage / "[df]", str(download)]
     )
     assert sorted(download.iterdir()) == [download / "[df]"]
 
     # Remove files with literal path
-    helper.run_cli(["storage", "rm", "--no-glob", helper.tmpstorage + "/[df]"])
+    helper.run_cli(["storage", "rm", "--no-glob", helper.tmpstorage / "[df]"])
     captured = helper.run_cli(["storage", "ls", helper.tmpstorage])
     assert sorted(captured.out.splitlines()) == ["d"]
 
@@ -593,10 +588,10 @@ def test_e2e_cp_filter(tmp_path: Path, helper: Helper) -> None:
             "--exclude",
             "*z",
             tmp_path.as_uri() + "/folder",
-            helper.tmpstorage + "/filtered",
+            helper.tmpstorage / "filtered",
         ]
     )
-    captured = helper.run_cli(["storage", "ls", helper.tmpstorage + "/filtered"])
+    captured = helper.run_cli(["storage", "ls", helper.tmpstorage / "filtered"])
     assert captured.out.splitlines() == ["bar"]
 
     # Copy all files to storage
@@ -606,7 +601,7 @@ def test_e2e_cp_filter(tmp_path: Path, helper: Helper) -> None:
             "cp",
             "-r",
             tmp_path.as_uri() + "/folder",
-            helper.tmpstorage + "/folder",
+            helper.tmpstorage / "folder",
         ]
     )
 
@@ -622,7 +617,7 @@ def test_e2e_cp_filter(tmp_path: Path, helper: Helper) -> None:
             "b??",
             "--exclude",
             "*z",
-            helper.tmpstorage + "/folder",
+            helper.tmpstorage / "folder",
             tmp_path.as_uri() + "/filtered",
         ]
     )
@@ -642,7 +637,7 @@ def test_e2e_ls_skip_hidden(tmp_path: Path, helper: Helper) -> None:
         ["storage", "cp", "-r", tmp_path.as_uri() + "/folder", helper.tmpstorage]
     )
 
-    captured = helper.run_cli(["storage", "ls", helper.tmpstorage + "/folder"])
+    captured = helper.run_cli(["storage", "ls", helper.tmpstorage / "folder"])
     assert captured.out.splitlines() == ["foo"]
 
 
@@ -659,7 +654,7 @@ def test_e2e_ls_show_hidden(tmp_path: Path, helper: Helper) -> None:
         ["storage", "cp", "-r", tmp_path.as_uri() + "/folder", helper.tmpstorage]
     )
 
-    captured = helper.run_cli(["storage", "ls", "--all", helper.tmpstorage + "/folder"])
+    captured = helper.run_cli(["storage", "ls", "--all", helper.tmpstorage / "folder"])
     assert captured.out.splitlines() == [".bar", "foo"]
 
 
